package cc.fascinated.platform;

import cc.fascinated.exception.impl.BadRequestException;
import lombok.AllArgsConstructor;
import lombok.Getter;

/**
 * @author Fascinated (fascinated7)
 */
@AllArgsConstructor
@Getter
public abstract class Platform {
    /**
     * The name of the platform.
     */
    private final Platforms platform;

    /**
     * Called every 10 minutes to update
     * the players data in QuestDB.
     */
    public abstract void updatePlayers();

    /**
     * Called every 10 minutes to update
     * the metrics for total scores, etc.
     */
    public abstract void updateMetrics();

    @AllArgsConstructor
    @Getter
    public enum Platforms {
        SCORESABER("scoresaber");

        /**
         * The internal name of the platform.
         */
        private final String platformName;

        /**
         * Gets a platform by its name.
         *
         * @param platform the name of the platform
         * @return the platform
         */
        public static Platforms getPlatform(String platform) {
            for (Platforms platforms : Platforms.values()) {
                if (platforms.getPlatformName().equalsIgnoreCase(platform)) {
                    return platforms;
                }
            }
            throw new BadRequestException("Platform '" + platform + "' was not found.");
        }
    }
}
