package cc.fascinated.platform.impl;

import cc.fascinated.model.score.TotalScoresResponse;
import cc.fascinated.model.token.ScoreSaberAccountToken;
import cc.fascinated.model.user.User;
import cc.fascinated.platform.Platform;
import cc.fascinated.services.QuestDBService;
import cc.fascinated.services.ScoreSaberService;
import cc.fascinated.services.TrackedScoreService;
import cc.fascinated.services.UserService;
import io.questdb.client.Sender;
import lombok.NonNull;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;


/**
 * @author Fascinated (fascinated7)
 */
@Component
public class ScoreSaberPlatform extends Platform {
    /**
     * The ScoreSaber service to use
     */
    @NonNull
    private final ScoreSaberService scoreSaberService;

    /**
     * The user service to use
     */
    @NonNull
    private final UserService userService;

    /**
     * The Influx service to use
     */
    @NonNull
    private final QuestDBService questDBService;

    /**
     * The tracked score service to use
     */
    @NonNull
    private final TrackedScoreService trackedScoreService;

    @Autowired
    public ScoreSaberPlatform(@NonNull ScoreSaberService scoreSaberService, @NonNull UserService userService, @NonNull QuestDBService questDBService,
                              @NonNull TrackedScoreService trackedScoreService) {
        super(Platforms.SCORESABER);
        this.scoreSaberService = scoreSaberService;
        this.userService = userService;
        this.questDBService = questDBService;
        this.trackedScoreService = trackedScoreService;
    }

    @Override
    public void updatePlayers() {
        for (User user : this.userService.getUsers()) {
            if (!user.isHasLoggedIn()) { // Check if the user has linked their account
                continue;
            }
            ScoreSaberAccountToken account = scoreSaberService.getAccount(user); // Get the account from the ScoreSaber API

            try (Sender sender = questDBService.getSender()) {
                sender.table("player")
                        .symbol("platform", this.getPlatform().getPlatformName())
                        .symbol("user_id", user.getId().toString())
                        .doubleColumn("pp", account.getPp())
                        .longColumn("rank", account.getRank())
                        .longColumn("country_rank", account.getCountryRank())
                        .longColumn("total_score", account.getScoreStats().getTotalScore())
                        .longColumn("total_ranked_score", account.getScoreStats().getTotalRankedScore())
                        .doubleColumn("average_ranked_accuracy", account.getScoreStats().getAverageRankedAccuracy())
                        .longColumn("total_play_count", account.getScoreStats().getTotalPlayCount())
                        .longColumn("ranked_play_count", account.getScoreStats().getRankedPlayCount())
                        .atNow();
            }
        }
    }

    @Override
    public void updateMetrics() {
        try (Sender sender = questDBService.getSender()) {
            TotalScoresResponse totalScores = trackedScoreService.getTotalScores(this.getPlatform());
            sender.table("metrics")
                    .symbol("platform", this.getPlatform().getPlatformName())
                    .longColumn("total_scores", totalScores.getTotalScores())
                    .longColumn("total_ranked_scores", totalScores.getTotalRankedScores())
                    .atNow();
        }
    }
}
