package cc.fascinated.services;

import cc.fascinated.exception.impl.BadRequestException;
import cc.fascinated.model.token.ScoreSaberAccountToken;
import cc.fascinated.model.user.User;
import kong.unirest.core.HttpResponse;
import kong.unirest.core.Unirest;
import org.springframework.stereotype.Service;

/**
 * @author Fascinated (fascinated7)
 */
@Service
public class ScoreSaberService {
    private static final String SCORESABER_API = "https://scoresaber.com/api/";
    private static final String GET_PLAYER_ENDPOINT = SCORESABER_API + "player/%s/full";

    /**
     * Gets the ScoreSaber account for a user.
     *
     * @param user the user to get the account for
     * @return the ScoreSaber account
     * @throws BadRequestException if an error occurred while getting the account
     */
    public ScoreSaberAccountToken getAccount(User user) {
        if (user.getSteamId() == null) {
            throw new BadRequestException("%s does not have a linked ScoreSaber account".formatted(user.getUsername()));
        }

        HttpResponse<ScoreSaberAccountToken> response = Unirest.get(GET_PLAYER_ENDPOINT.formatted(user.getSteamId()))
                .asObject(ScoreSaberAccountToken.class);

        if (response.getParsingError().isPresent()) { // Failed to parse the response
            throw new BadRequestException("Failed to parse ScoreSaber account for %s".formatted(user.getUsername()));
        }
        if (response.getStatus() != 200) { // The response was not successful
            throw new BadRequestException("Failed to get ScoreSaber account for %s".formatted(user.getUsername()));
        }
        return response.getBody();
    }
}
